var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
import Clutter from 'gi://Clutter';
import GObject from 'gi://GObject';
import Shell from 'gi://Shell';
import St from 'gi://St';
import WidgetBase from './WidgetBase.js';
import { getWeekdayAbbr, getMonthName, daysInMonth, sameDay } from './dateUtils.js';
let Calendar = class Calendar extends WidgetBase {
    LOCATION_SETTING_KEY = 'calendar-location';
    _weekStart;
    _calendar;
    _draggable;
    _topBox;
    _monthLabel;
    _firstDayIndex;
    _selectedDate;
    _buttons;
    isDragging;
    _dragMonitor;
    startX;
    startY;
    oldX;
    oldY;
    rowHeight;
    rowWidth;
    deltaX;
    deltaY;
    ignoreUpdatePosition;
    constructor(settings) {
        super({
            reactive: true,
        });
        this._weekStart = Shell.util_get_week_start();
        this._settings = settings;
        this._calendar = new St.Widget({
            style_class: 'calendar-wd',
            layout_manager: new Clutter.GridLayout(),
            reactive: true,
        });
        this._settings.connect('changed::hide-calendar-widget', () => this._toggleShow());
        this._settings.connect('changed::calendar-location', () => this.setPosition());
        // // this._draggable = DND.makeDraggable(this)
        // this._draggable = DND.makeDraggable(this, undefined)
        // // @ts-ignore
        // this._draggable._animateDragEnd = eventTime => {
        //   // @ts-ignore
        //   this._draggable._animationInProgress = true
        //   // @ts-ignore
        //   this._draggable._onAnimationComplete(this._draggable._dragActor, eventTime)
        // }
        // this._draggable.connect('drag-begin', this._onDragBegin.bind(this))
        // this._draggable.connect('drag-end', this._onDragEnd.bind(this))
        this._toggleShow();
        this.setPosition();
    }
    _toggleShow() {
        this.remove_all_children();
        if (!this._settings.get_boolean('hide-calendar-widget')) {
            this.add_child(this._calendar);
        }
        this._buildHeader();
        this.update();
    }
    _buildHeader() {
        // Top line of the calendar '<| September 2009 |>'
        this._topBox = new St.BoxLayout({});
        this._calendar.layout_manager.attach(this._topBox, 0, 0, 7, 1);
        this._monthLabel = new St.Label({
            style_class: 'calendar-header-label',
            can_focus: true,
            x_align: Clutter.ActorAlign.CENTER,
            x_expand: true,
            y_align: Clutter.ActorAlign.CENTER,
        });
        this._topBox.add_child(this._monthLabel);
        // Add weekday labels...
        for (let i = 0; i < 7; i++) {
            let label = new St.Label({
                style_class: 'weekday-label',
                text: getWeekdayAbbr(i),
            });
            this._calendar.layout_manager.attach(label, i, 1, 1, 1);
        }
        // All the children after this are days, and get removed when we update the calendar
        this._firstDayIndex = this._calendar.get_n_children();
    }
    update() {
        this._selectedDate = new Date();
        if (!this._monthLabel) {
            throw new TypeError('_mothLabel was not initialized correctly');
        }
        this._monthLabel.text = getMonthName(this._selectedDate.getMonth());
        let now = new Date();
        let children = this._calendar.get_children();
        if (!this._firstDayIndex) {
            throw new TypeError('_firstDayIndex was not initialized correctly');
        }
        for (let i = this._firstDayIndex; i < children.length; i++) {
            children[i].destroy();
        }
        this._buttons = [];
        let totalDays = daysInMonth(this._selectedDate.getMonth(), this._selectedDate.getFullYear());
        let firstDay = new Date(this._selectedDate.getFullYear(), this._selectedDate.getMonth(), 1);
        let row = 2;
        for (let i = 0; i < totalDays; i++) {
            let dateLabel = new St.Button({
                // label: firstDay.toLocaleFormat('%d'),
                label: firstDay.getDate().toString(),
            });
            let styleClass = 'day-base';
            if (sameDay(now, firstDay)) {
                styleClass += ' today';
            }
            dateLabel.style_class = styleClass;
            this._calendar.layout_manager.attach(dateLabel, firstDay.getDay(), row, 1, 1);
            if (firstDay.getDay() == 6) {
                row++;
            }
            firstDay.setDate(firstDay.getDate() + 1);
        }
    }
};
Calendar = __decorate([
    GObject.registerClass
], Calendar);
export { Calendar };
export default Calendar;
