import GLib from 'gi://GLib';
import { Extension } from 'resource:///org/gnome/shell/extensions/extension.js';
import * as Main from 'resource:///org/gnome/shell/ui/main.js';
import Calendar from './Calendar.js';
import Clock from './Clock.js';
import Cpu from './Cpu.js';
import NetSpeed from './NetSpeed.js';
import Ram from './Ram.js';
export default class CircularWidgetExtension extends Extension {
    _calendar;
    _clock;
    _cpu;
    _netSpeed;
    _ram;
    _settings;
    _timeoutId;
    _layoutManager;
    enable() {
        const settings = this._settings = this.getSettings();
        const calendar = this._calendar = new Calendar(settings);
        const clock = this._clock = new Clock(settings);
        const cpu = this._cpu = new Cpu(settings);
        const netSpeed = this._netSpeed = new NetSpeed(settings);
        const ram = this._ram = new Ram(settings);
        const layoutManager = this._layoutManager = Main.layoutManager;
        layoutManager._backgroundGroup.add_child(calendar);
        layoutManager._backgroundGroup.add_child(clock);
        layoutManager._backgroundGroup.add_child(cpu);
        layoutManager._backgroundGroup.add_child(netSpeed);
        layoutManager._backgroundGroup.add_child(ram);
        this._timeoutId = GLib.timeout_add(GLib.PRIORITY_DEFAULT, 500, () => {
            calendar.update();
            clock.update();
            cpu.update();
            netSpeed.update();
            ram.update();
            return GLib.SOURCE_CONTINUE;
        });
    }
    disable() {
        this._settings = null;
        if (this._timeoutId) {
            GLib.Source.remove(this._timeoutId);
            this._timeoutId = null;
        }
        this._layoutManager._backgroundGroup.remove_child(this._calendar);
        this._layoutManager._backgroundGroup.remove_child(this._clock);
        this._layoutManager._backgroundGroup.remove_child(this._cpu);
        this._layoutManager._backgroundGroup.remove_child(this._netSpeed);
        this._layoutManager._backgroundGroup.remove_child(this._ram);
        this._layoutManager = null;
        this._calendar = null;
        this._clock = null;
        this._cpu = null;
        this._netSpeed = null;
        this._ram = null;
    }
}
